#ifndef __Graphics
#define __Graphics

#include <SDL.h>
//#include <cstdlib>
#include <iostream>
         
//using namespace std;

class Graphics {

 private:
  SDL_Surface *screen;
  Uint32 color;

  void Slock();
  void Sulock();


 public:
  int winWidth, winHeight;

  Graphics();
  void openWindow(const char* title,int _width, int _height);
  void setColor(Uint8 R, Uint8 G, Uint8 B);
  void flip();
  void Clear(const char& g);

  SDL_Surface *getSurface();

  Uint32 getpixel(int x, int y);
  void pixel(int x, int y,  const char& r, const char& g, const char&b); 
  void largepixel(int x, int y,  const char& r, const char& g, const char&b); 
  void largestpixel(int x, int y,  const char& r, const char& g, const char&b); 
  void addpixel(const int& x, const int& y, const char& r, const char& g, const char& b);
  void pixel(int x, int y, Uint32 p);
  void line(int x1, int y1, int x2, int y2, const char& r, const char& g, const char& b);
  bool Verify(const int& x, const int&y); 
  bool Verify2(const int& x, const int&y); 

  SDL_Surface* CreateSurface();

	void Copy(SDL_Surface* from, SDL_Surface* too);
  static Uint32 RGB(const char& r, const char& g, const char&b)
    {
      return ((int)r) | ((int)g)<<8 | ((int)b)<<16;
    }

};


/**********************************************************************/

SDL_Surface* Graphics::getSurface()
{
 return screen;
 }                   

Graphics::Graphics() {

  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
    exit(1);
  }
  atexit(SDL_Quit);

  winWidth = winHeight = 0;
  screen = NULL;
}


void Graphics::Slock() {
  if ( SDL_MUSTLOCK(screen) )
    if ( SDL_LockSurface(screen) < 0 )
      return;
}


void Graphics::Sulock() {
  if ( SDL_MUSTLOCK(screen) )
    SDL_UnlockSurface(screen);
}


SDL_Surface* Graphics::CreateSurface()
{
	return SDL_CreateRGBSurface(SDL_HWSURFACE|SDL_DOUBLEBUF, winWidth, winHeight, 32,
                          0,0,0,0);
}

void Graphics::openWindow(const char* title, int _width, int _height) {
  winWidth = _width;
  winHeight = _height;
  screen = SDL_SetVideoMode(_width, _height, 32,
			    SDL_HWSURFACE|SDL_DOUBLEBUF);
	SDL_WM_SetCaption(title,title); 
  if ( screen == NULL ) {
    printf("Unable to set video: %s\n", SDL_GetError());
    exit(1);
  }
  Slock();
}



void Graphics::setColor(Uint8 R, Uint8 G, Uint8 B) {
  if ( screen == NULL ) {
    printf("Unable to set color. Screen not set.\n");
    exit(1);
  }
  color = SDL_MapRGB(screen->format, R, G, B);
}

void Graphics::Clear(const char& g )
{
  memset(screen->pixels, g,winWidth*winHeight*4);
}

void Graphics::flip() {
  Sulock();
  SDL_Flip(screen);
  Slock();
}

void Graphics::Copy(SDL_Surface* from, SDL_Surface* too) {
  SDL_Rect src, dst;
  src.x = 0;  src.y = 0;  src.w = winWidth;  src.h = winHeight;
  dst.x = 0;  dst.y = 0;  dst.w = src.w;  dst.h = src.h;
  //if (alpha != 255) SDL_SetAlpha(font, SDL_SRCALPHA, alpha);
  SDL_BlitSurface(from, &src, too, &dst);

}

Uint32 Graphics::getpixel(int x, int y) {
  Uint32 a = *((Uint32 *)screen->pixels + y*screen->pitch/4 + x);
  return a;
}

void Graphics::addpixel(const int& x, const int& y, const char& r, const char& g, const char &b)
{
  if (!Verify(x,y))
    return;
  Uint32* pos = (Uint32 *)screen->pixels + y*screen->pitch/4 + (int)x;
  Uint32 _b = (((*pos) & 255) +  (int)b); if (_b>255) _b = 255;
  Uint32 _g = (((*pos)>>8 & 255) + (int)g);if (_g>255) _g = 255;
  Uint32 _r = (((*pos)>>16 & 255) + (int)r);if (_r>255) _r = 255;
  *pos = _b | (_g<<8) | (_r<<16);

}

bool Graphics::Verify(const int&x, const int&y)
{
  if ((x>=winWidth) || (x<0)) return false;
  if ((y>=winHeight) || (y<0)) return false;
  return true;
}
bool Graphics::Verify2(const int&x, const int&y)
{
  if ((x>=winWidth-1) || (x<1)) return false;
  if ((y>=winHeight-1) || (y<1)) return false;
  return true;
}


void Graphics::pixel(int x, int y, const char& r, const char& g, const char&b) {
  if (!Verify(x,y))
    return;
  Uint32 *bufp  = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
  *bufp = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
}

void Graphics::largepixel(int x, int y, const char& r, const char& g, const char&b) {
  if (!Verify2(x,y))
    return;
  Uint32 *bufp  = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
  *bufp = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
  *(bufp+1) = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
}

void Graphics::largestpixel(int x, int y, const char& r, const char& g, const char&b) {
  if (!Verify2(x,y))
    return;
  Uint32 *bufp  = (Uint32 *)screen->pixels + y*screen->pitch/4 + x;
  *bufp = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
  *(bufp+1) = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
  *(bufp-1) = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
  *(bufp-screen->pitch/4) = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);
  *(bufp+screen->pitch/4) = SDL_MapRGB(screen->format, r, g, b); //RGB(r,g,b);

}

void Graphics::line(int x1, int y1, int x2, int y2, const char& r, const char& g, const char&b) {
  int xStart, xEnd, yStart, yEnd, dx, dy;
  double slope;
  if (x1 < x2) {
    xStart = x1;
    xEnd   = x2;
    yStart = y1;
    yEnd   = y2;
  }
  else {
    xStart = x2;
    xEnd   = x1;
    yStart = y2;
    yEnd   = y1;
  }

  dx = xEnd - xStart;
  dy = yEnd - yStart;

  if (abs(dy) < dx) {
    slope = dy/double(dx);
    for (int x = xStart; x <= xEnd; x++)
      pixel(x, yStart+int(slope*(x-xStart)),r,g,b);
  }
  else if (dy >= 0) {
    slope = dx/double(dy);
    for (int y = yStart; y <= yEnd; y++)
      pixel(xStart + int(slope*(y-yStart)), y,r,g,b);
  }
  else {
    slope = dx/double(dy);
    for (int y = yStart; y >= yEnd; y--)
      pixel(xStart + int(slope*(y-yStart)), y,r,g,b);
  }

  //double slope = (yEnd-yStart)/double(xLarge-xSmall);

  //for (int x = xSmall; x <= xLarge; x++)
  //  pixel(x, yStart + int(slope*(x-xSmall)) );

}


#endif
