#include "Graphics.h"
#include <math.h>

Graphics g;
int W = 800; // Width
int H = 300;  // Height

/*
  Partial differential equation: Solving the wave equation
  Nicolaas E. Groeneboom, FYS4150 H06
  leuat@irio.co.uk
  http://www.irio.co.uk
*/

int main(int a, char *b[])
{
  // Opens a window
  g.openWindow("Wave schmave",W,H);
  
  // Allocate 3 buffers: the solution, 1st and 2nd differentiated solutions
  double* w1 = new double[W];  double* w2 = new double[W];  double* w3 = new double[W];
  double* tmp = 0;  double* wa = w1;  double* wb = w2;  double* wc = w3;
  
  // Wavenumber: must be <1 in order to be stable. Experiment with k, let  
  // k(x) be dependent on x.. looks nice. Notice the reflection of the wave where k changes.
  double k = 0.9;

  // This boy generates a gaussian clock-wave (x = c*exp(-x^2), gaussian distribution)
  for (int i=0;i<W;i++)
    {
      double x = -(4.0) + i*(20.0 / W);
      double y = -(12.0) + i*(25.0 / W);
      wa[i] = exp(-x*x)*1.5 + y*exp(-y*y*0.7)*2.5;
      wb[i] = 0.9*wa[i];
      wc[i] = 0.9*wa[i];
    }

  // This variable maintains events
  SDL_Event event;

  // run forever with me!
  bool quit = false;
  while(!quit)
    {
      // Clears the screen (0=black, 255=white)
      g.Clear(0);

      // Draws the solution, pixel by pixel...
      for (int i=0;i<W;i++)
	      g.pixel(i,H/2-(int)(wb[i]*70.0),100,255,100);


      // The main routine that calculates the next solution
      for (int i=1;i<W-1;i++)  {

	// if you uncomment the following line, the wave will propagate faster for x>half the width... (variable k)
	//if (i>W/2) k = 0.9; else k=0.2;
	
	wa[i] = k* (wb[i-1] - 2* wb[i] + wb[i+1]) - wc[i] + 2*wb[i]; 
      }
      
      // Nice trick: permute the solutions to save memory
      tmp = wc; wc = wb; wb=wa; wa=tmp;
      
      // Present to the screen
      g.flip();
      
      // Check if keys are pressed
      while ( SDL_PollEvent(&event) ) {
	        switch (event.type) {
 	              case SDL_QUIT:
	                   quit=true;
                  case SDL_KEYDOWN:
	                   quit=true; // guess what happens here
	  
            }   
      }
    }

  // Clean up that memory!
  delete[] w1;  delete[] w2;  delete[] w3; 
  return 0;
}
